import base64
import logging

from yambot.router import Router
from time import sleep
from requests import post

from yambot.types import Update

API_URL = 'https://botapi.messenger.yandex.net/bot/v1/messages'

class MessengerBot(Router):
    def __init__(self, token, log_level=logging.INFO):
        super().__init__(log_level=log_level)
        self._token = token
        self._headers = {'Authorization': f'OAuth {token}', 'Content-Type': 'application/json'}


    def start_pooling(self):
        self._logger.info('Starting pooling...')
        last_update_id = -1

        while True:
            try:
                request_body = {'limit': 10, 'offset': last_update_id + 1}

                response = post(f'{API_URL}/getUpdates', json=request_body, headers=self._headers)
                if response.status_code != 200:
                    raise ConnectionError(f'{response.status_code} - {response.text}')
                response_json = response.json()

                if 'updates' in response_json:
                    updates = response_json['updates']

                    if len(updates) > 0:
                        last_update_id = int(updates[len(updates) - 1]['update_id'])

                        for update in updates:
                            self._logger.debug(f'Got update: {update}')
                            update_obj = Update.from_dict(update)
                            self._process_update(update_obj)
            except Exception as e:
                self._logger.error(f'Error while pooling: {e}')
            finally:
                sleep(1)

    def _send_text(self, body, update: Update):
        path = f'{API_URL}/sendText'

        if update.chat.chat_type == 'group':
            if update.chat.thread_id and update.chat.thread_id != '0':
                body.update({'chat_id': update.chat.chat_id, 'thread_id': update.chat.thread_id})
            else:
                body.update({'chat_id': update.chat.chat_id})
        else:
            body.update({'login': update.from_m.login})
        self._logger.debug(f'Sending text: {body}')
        try:
            response = post(path, json=body, headers=self._headers)
            if response.status_code != 200:
                raise ConnectionError(f'{response.status_code} - {response.text}')
            self._logger.debug(f'Send message response: {response.text}')
            return response.json()
        except Exception as e:
            self._logger.error(f'Error while sending message: {e}')
            return {}

    def _send_image_form(self, files, update: Update, path = f'{API_URL}/sendImage'):

        headers = {'Authorization': f'OAuth {self._token}'}
        body = {}

        if update.chat.chat_type == 'group':
            if update.chat.thread_id and update.chat.thread_id != '0':
                body.update({'chat_id': update.chat.chat_id, 'thread_id': update.chat.thread_id})
            else:
                body.update({'chat_id': update.chat.chat_id})
        else:
            body.update({'login': update.from_m.login})
        try:
            response = post(path, headers=headers, files=files, data=body)
            if response.status_code !=200:
                raise ConnectionError(f'{response.status_code} - {response.text}')
            self._logger.debug(f'Send image response: {response.text}')
            return response.json()
        except Exception as e:
            self._logger.error(f'Error while sending image: {e}')
            return {}


    def send_message(self, text, update: Update, disable_web_page_preview = True):
        body = {'text': text, 'disable_web_page_preview': disable_web_page_preview}
        return self._send_text(body, update)

    def delete_message(self, update: Update):
        path = f'{API_URL}/delete/'
        body = {'message_id': update.message_id}

        if update.chat.chat_type == 'group':
            if update.chat.thread_id and update.chat.thread_id != '0':
                body.update({'chat_id': update.chat.chat_id, 'thread_id': update.chat.thread_id})
            else:
                body.update({'chat_id': update.chat.chat_id})
        else:
            body.update({'login': update.from_m.login})
        self._logger.debug(f'Delete request: {body}')
        response = post(path, json=body, headers=self._headers)
        self._logger.debug(f'Delete response: {response.status_code}')
        return response.status_code

    def send_inline_keyboard(self, text, buttons: [], update: Update):
        body = {'text': text, 'inline_keyboard': buttons}
        return self._send_text(body, update)

    def send_image(self, image, update: Update):
        try:
            img_data = base64.b64decode(image)
        except TypeError:
            img_data = image
        files = [('image', ('image.jpeg', img_data, 'image/jpeg'))]
        self._send_image_form(files, update)

    def send_gallery(self, images: [], update: Update):
        files = []
        index = 0
        for image in images:
            try:
                img_data = base64.b64decode(image)
            except TypeError:
                img_data = image
            files.append(('images', (f'image{index}.jpeg', img_data, 'image/jpeg')))
            index += 1
        self._send_image_form(files, update, path=f'{API_URL}/sendGallery')


import json


class _BaseObject(object):
    def to_json(self):
        return json.dumps(self)


class Chat:
    def __init__(self, chat_type, chat_id, thread_id):
        self._chat_id = chat_id
        self._chat_type = chat_type
        self._thread_id = thread_id

    @classmethod
    def from_dict(cls, obj: dict):
        return cls(
            chat_id=obj.get('id', None),
            chat_type=obj['type'],
            thread_id=obj.get('thread_id', None)
        )

    @property
    def chat_id(self):
        return self._chat_id

    @property
    def chat_type(self):
        return self._chat_type

    @property
    def thread_id(self):
        return self._thread_id


class From:
    def __init__(self, display_name, from_id, login, robot):
        self._display_name = display_name
        self._from_id = from_id
        self._login = login
        self._robot = robot

    @classmethod
    def from_dict(cls, obj: dict):
        return cls(
            display_name=obj['display_name'],
            from_id=obj['id'],
            login=obj['login'],
            robot=obj['robot']
        )

    @property
    def display_name(self):
        return self._display_name

    @property
    def from_id(self):
        return self._from_id

    @property
    def login(self):
        return self._login

    @property
    def robot(self):
        return self._robot

class Update:
    def __init__(self, chat: Chat, from_m: From, message_id, text, timestamp,
                 update_id, reply_to_message, callback_data):
        self._chat = chat
        self._from_m = from_m
        self._message_id = message_id
        self._text = text
        self._timestamp = timestamp
        self._update_id = update_id
        self._reply_to_message = reply_to_message
        self._callback_data = callback_data

    @classmethod
    def from_dict(cls, obj: dict):
        reply_to_message = None
        if 'reply_to_message' in obj:
            reply_to_message = Update.from_dict(obj['reply_to_message'])
        return cls(
            chat=Chat.from_dict(obj['chat']),
            from_m=From.from_dict(obj['from']),
            message_id=obj['message_id'],
            text=obj['text'],
            timestamp=obj['timestamp'],
            update_id=obj['update_id'],
            reply_to_message=reply_to_message,
            callback_data=obj.get('callback_data', None)

        )

    @property
    def chat(self):
        return self._chat

    @property
    def from_m(self):
        return self._from_m

    @property
    def message_id(self):
        return self._message_id

    @property
    def text(self):
        return self._text

    @property
    def timestamp(self):
        return self._timestamp

    @property
    def update_id(self):
        return self._update_id

    @property
    def reply_to_message(self):
        return self._reply_to_message

    @property
    def callback_data(self):
        return self._callback_data

